#ifndef DEVICEDHT_H
#define DEVICEDHT_H

#include "Device.h"
#include <Homie.h>
#include <DHT.h>    // https://github.com/adafruit/DHT-sensor-library

class DeviceDht : public Device {
  public:
    inline DeviceDht(byte dhtPin, byte dhtType):dht(dhtPin, dhtType) {
      pin = dhtPin;
      type = dhtType;
    }
    virtual void setup();
    virtual void homieRegister();
    virtual void loop();
  private:
    byte pin;
    byte type;
    DHT dht;
    const int INTERVAL_DHT = 60;
    unsigned long lastSentDHT = 0;
    float humidity, temperature;  // raw values from the sensor
    float heatindex;              // computed value from the sensor
    HomieNode humidityNode{"humidity", "humidity"};
    HomieNode temperatureNode{"temperature", "temperature"};
    HomieNode heatindexNode{"heatindex", "heatindex"};
};

void DeviceDht::setup() {
  pinMode(pin, INPUT);
  dht.begin();
}

void DeviceDht::homieRegister() {
  Homie.registerNode(humidityNode);
  Homie.registerNode(temperatureNode);
  Homie.registerNode(heatindexNode);
}

// compare float values
bool isEqual(float a, float b, float epsilon=0.001) {
  return fabs(a - b) <= epsilon * fabs(a);
}

void DeviceDht::loop() {
  if (millis() - lastSentDHT >= INTERVAL_DHT * 1000UL || lastSentDHT == 0) {
    float previousHumidity = humidity;
    float previousTemperature = temperature;
    float previousHeatindex = heatindex;
    humidity = dht.readHumidity();        // read humidity as a percent
    temperature = dht.readTemperature();  // read temperature as Celsius
    heatindex = dht.computeHeatIndex(temperature, humidity, false);

    // check if any reads failed and exit early
    if (isnan(humidity) || isnan(temperature)) {
      Serial.println("Failed to read from DHT sensor!");
      return;
    }

    if (!isEqual(humidity, previousHumidity)) {
      Serial.print("humidity: ");
      Serial.println(humidity);
      if (!Homie.setNodeProperty(humidityNode, "value", String(humidity), true)) {
        Serial.println("Sending failed");
      }
    } else {
      Serial.println("humidity unchanged");
    }

    if (!isEqual(temperature, previousTemperature)) {
      Serial.print("temperature: ");
      Serial.println(temperature);
      if (!Homie.setNodeProperty(temperatureNode, "value", String(temperature), true)) {
        Serial.println("Sending failed");
      }
    } else {
      Serial.println("temperature unchanged");
    }

    if (!isEqual(heatindex, previousHeatindex)) {
      Serial.print("heatindex: ");
      Serial.println(heatindex);
      if (!Homie.setNodeProperty(heatindexNode, "value", String(heatindex), true)) {
        Serial.println("Sending failed");
      }
    } else {
      Serial.println("heatindex unchanged");
    }

    lastSentDHT = millis();
  }
}

#endif
